use strict;
use warnings;

use Test::More tests => 1;

use_ok('Mac::Apps::Seasonality::Constants');

# setup vim: set filetype=perl tabstop=4 softtabstop=4 expandtab :
# setup vim: set shiftwidth=4 shiftround textwidth=0 nowrap autoindent :
# setup vim: set foldmethod=indent foldlevel=0 :
